#!/bin/bash
GENERATOR=${GENERATOR:='adfGenerator'}
CLINGO=${CLINGO:='./clingo'}
# default parameters
PARAS=${PARAS:=10}                     # number of statements
PARASE=${PARASE:=0.5}                  # probability of symmetric edges
PARAC=${PARAC:=0.1}                    # probability of constants
PARAD=${PARAD:=0.5}                    # probability of disjunctions instead of conjunctions
PARATW=${PARATW:=7}                    # width of 'grid'

usage() {
  echo "usage: $0 numberOfStatements"
  echo "viewing README.txt: $0 -h"
}

stringerror() {
  echo "ERROR: numberOfStatements must be a positive integer"
}

if [ "$1" == "-h" ]; then
  cat README.txt
  exit 0
fi

# check if generator is compiled (only _very_ simple check!)
if [ ! -f $GENERATOR.class ]; then
  javac $GENERATOR.java
  if [ $? != 0 ]; then
    echo "ERROR: An error occured when compiling the generator (javac)"
    exit 1
  fi
fi

# extract arguments
PARAS=$1
if [ "$PARAS" == "" ]; then
  usage
  exit 1
fi

# check if PARAS is a number
case $PARAS in
  ''|*[!0-9]*) stringerror
  usage
  exit 1 ;;
  *) ;;
esac

# check if PARAS is at least one
if [ $PARAS -lt 1 ]; then
  stringerror
  usage
  exit 1
fi 

# generate ADF
# 1. call the java program to generate an "annotated" AF (with grid-structure)
# 2. give the result to clingo with the ASP module transform the whole stuff to an ADF
# tail/head/sed/awk just extract the answer-set from clingo and transforms it to new facts (i.e. add trailing ".")
java $GENERATOR $PARAS $PARATW $PARASE $PARAC $PARAD | $CLINGO - constructFormula.dl 1 2> /dev/null | tail -n 8 | head -n 1 | sed "s/v([0-9]*)/c(v)/g" | sed "s/f([0-9]*)/c(f)/g" | awk -f as2kb.awk
